const glob = require('glob')
const jpa = require('js-jpa')
const TaskManager = require('../task/TaskManager')
const HttpServer = require('../net/HttpServer')

module.exports = class InternalContext {
  async initialize(config) {
    this._config = config
    this._taskManager = new TaskManager(this)
    this._httpServer = new HttpServer(this._config.appPort, this._config.appStatic)
    if (this._config.database) {
      this._dataSource = await jpa.newDataSource(this._config.database)
    }
  }

  async startup() {
    await this._httpServer.startup()
  }

  async shutdown() {
    await this._httpServer.shutdown()
    if (this._dataSource) {
      this._dataSource.release()
    }
  }

  async createSqlContext() {
    return await this._dataSource.createContext()
  }

  async scan(path, filter) {
    path = this._config.resolvePath(path)
    let files = glob.sync(path)
    if (files.length > 0) {
      for (let i = 0; i < files.length; i++) {
        let mod = require(files[i])
        if (filter) {
          await filter(mod)
        }
      }
    }
  }

  async installRepositories(path) {
    if (this._config.database) {
      let context = await this._dataSource.createContext()
      await this.scan(path, async (mod) => {
        await this._dataSource.registerRepository(context, mod)
      })
      context.release()
    } else {
      throw new Error('not found database config')
    }
  }

  async installRepository(context, repository) {
    await this._dataSource.registerRepository(context, repository)
  }

  installController(controller) {
    controller.handlers.forEach((handler) => {
      let url = controller.path + handler.path
      let type = handler.type || 'http'
      let method = handler.method || 'get'
      let isTransaction = handler.transaction === undefined ? (method !== 'get') : handler.transaction
      if (type === 'socket') {
        // socketRouter.register(method, url, _taskManager.createWebSocketTask(handler.handler, isTransaction))
      } else {
        this._httpServer.addTask(method, url, this._taskManager.createHttpTaskAdaptor(handler, isTransaction), handler.files)
      }
    })
  }

  schedule(task, isTransaction) {
    this._taskManager.schedule(task, isTransaction)
  }
}
